package cp;

import cp.db.DB;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

public class Club implements Comparable {
  private int index = -1;
  private String name;
  private Vector<Player> players = new Vector<Player>();
  private int chess4members = 0;

  public Club() {}

  public void setChess4Members(int chess4members) {
    this.chess4members = chess4members;
  }
  
  public void add(Player player) {
    players.add(player);
  }
  
  public double getScore(Round round) {
    double ans = 0;
    for (int i=0; i<players.size(); i++) {
      ans += players.get(i).getScore(round);
    }
    return ans;
  }

  public int getChess4Score(Round round) {
    double points = 0;
    for (int i=0; i<players.size(); i++) {
      points += players.get(i).getScore(round);
    }
    // Calculate club score according to rules
    double members = Math.max(10, chess4members);
    return (int)Math.round(40.0/members*points);
  }

  public String toString() {
    return name;
  }

  public int getChess4Members() {
    return chess4members;
  }
  
  protected void load(ResultSet rs) throws SQLException {
    index = rs.getInt("index");
    name = rs.getString("club");
    chess4members = rs.getInt("chess4members");
  }
  
  public void update() throws SQLException {
    Connection conn = DB.getConnection();
    String sql = "UPDATE clubs " + "SET club=?, chess4members=?" + "WHERE index=?";
    PreparedStatement s = conn.prepareStatement(sql);
    s.setString(1, name);
    s.setInt(2, chess4members);
    s.setInt(3, index);
    try {
      int rows = s.executeUpdate();
      if (rows != 1) {
        throw new SQLException(rows + " rader ndrades, frvntade 1 rad");
      }
      conn.commit();
    }
    catch (SQLException e) {
      conn.rollback();
      throw e;
    }
    s.close();
  }
  
  public void save() throws SQLException {
    Connection conn = DB.getConnection();
    String sql = "INSERT INTO clubs " + 
                 "(club, chess4members) " + 
                 "VALUES (?,?)";
    PreparedStatement s = conn.prepareStatement(sql, Statement.RETURN_GENERATED_KEYS);
    s.setString(1, name);
    s.setInt(2, chess4members);
    try {
      int rows = s.executeUpdate();
      if (rows != 1) {
        throw new SQLException(rows + " rader ndrades, frvntade 1 rad");
      }
      // Read the generated index
      ResultSet rs = s.getGeneratedKeys();
      rs.next();
      index = rs.getInt(1);
      conn.commit();
    }
    catch (SQLException e) {
      conn.rollback();
      throw e;
    }
    s.close();
  }

  int getIndex() {
    return index;
  }

  void clearPlayers() {
    players.clear();
  }
  
  /**
   * Compares players according to rating according to ratingN or tournament settings (if no rating exist, the name).
   */
  public int compareTo(Object o) {
    Club b = (Club) o;
    return name.compareTo(b.name);
  }

  public void setName(String name) {
    this.name = name;
  }
}
