package cp.gui;

import cp.Club;
import cp.Round;

import java.sql.SQLException;

import java.util.Collections;
import java.util.Vector;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;

public class Chess4StandingsTableModel extends AbstractTableModel {
  private String[] columnNames = { "Plac", "Klass", "Spelare", "Klasstorlek", "Pong" };
  private Round round;
  private Chess4StandingsComparator chess4StandingsComparator;
  private Vector<Club> currentClubs = new Vector<Club>();

  void setRound(Round round, Vector<Club> currentClubs) {
    this.round = round;
    this.currentClubs = currentClubs;
    update();
  }

  public String getColumnName(int col) {
    return columnNames[col].toString();
  }

  public int getRowCount() {
    if (round != null) {
      return currentClubs.size();
    }
    else {
      return 2;
    }
  }

  public int getColumnCount() {
    return columnNames.length;
  }

  public Object getValueAt(int rowIndex, int columnIndex) {
    if (round != null) {
      Collections.sort(currentClubs, chess4StandingsComparator);
			Club club = currentClubs.get(rowIndex);
      if (columnIndex == 0) {
        if (rowIndex > 0) {
					Club previousClub = currentClubs.get(rowIndex - 1);
          if (chess4StandingsComparator.compare(club, previousClub) == 0) {
            return null;
          }
        }
        return rowIndex + 1;
      }
      else if (columnIndex == 1) {
        return club;
      }
      else if (columnIndex == 2) {
        return round.getTournament().getPlayers().playsIn(club);
      }
      else if (columnIndex == 3) {
        return club.getChess4Members();
      }
      else if (columnIndex == 4) {
        return club.getChess4Score(round);
      }
    }
    return null;
  }

  public boolean isCellEditable(int row, int col) {
    return false;
  }

	Round getRound() {
    return round;
  }

  public void update() {
    if (round != null) {
      chess4StandingsComparator = new Chess4StandingsComparator(round);
    }
    else {
      chess4StandingsComparator = new Chess4StandingsComparator(null);
    }
    fireTableStructureChanged();
    fireTableDataChanged();
  }
}
