package cp.gui;

import cp.*;
import java.sql.SQLException;
import java.util.*;
import javax.swing.table.AbstractTableModel;

public class EditResultsTableModel extends AbstractTableModel {
  Vector<String> columnNames = new Vector<String>();
  private Round round;
  private Players players;
  private int resultColumn;

  EditResultsTableModel() {
    initColumnNames();
  }

  void setRound(Round round) {
    this.round = round;
    update();
  }
  
  public String getColumnName(int col) {
    return columnNames.get(col);
  }

  public int getRowCount() {
    if (round != null) {
      return round.getGames().size();
    }
    else {
      return 2;
    }
  }

  public int getColumnCount() {
    return columnNames.size();
  }

  public Object getValueAt(int rowIndex, int columnIndex) {
    if (round != null) {
			Game game = getGame(rowIndex);
      int columns = 6;
      if (round.getTournament().getShowELO()) {
        columns = columns+2;
      }
      if (getColumnName(columnIndex).equalsIgnoreCase("Bord")) {
        return rowIndex+1;
      }
      else if (getColumnName(columnIndex).equalsIgnoreCase("Vit spelare")) {
        return (game.getWhitePlayer() != null ? game.getWhitePlayer() : "frirond");
      }
      else if (getColumnName(columnIndex).equalsIgnoreCase("Resultat")) {
        return game.getResult();
      }
      else if (getColumnName(columnIndex).equalsIgnoreCase("Rating") && columnIndex <= 3) {
        Player white = game.getWhitePlayer();
        if (white != null && white.getRating() > 0) {
          return white.getRating();
        }
        return "";
      }
      else if (getColumnName(columnIndex).equalsIgnoreCase("Rating") && columnIndex > 3) {
        Player black = game.getBlackPlayer();
        if (black != null && black.getRating() > 0) {
          return black.getRating();
        }
        return "";
      }
      else if (getColumnName(columnIndex).equalsIgnoreCase("Svart spelare")) {
        return (game.getBlackPlayer() != null ? game.getBlackPlayer() : "frirond");
      }
    }
    return null;
  }
  
  public Game getGame(int rowIndex) {
    if (round != null) {
			Games games = round.getGames();
			Game game = games.get(rowIndex);
      return game;
    }    
    return null;
  }
  
  public boolean isCellEditable(int row, int col) {
    return false;
  }

  void delete(int row) throws SQLException {
		Game game = getGame(row);
    if (game != null) {
      game.getGames().delete(round, game);
    }
  }

	Game appendPair() throws SQLException, ChessPairerException {
    if (round != null) {
			Game game = new Game(round.getGames());
      round.append(game);
      return game;
    }
    return null;
  }

	Round getRound() {
    return round;
  }
  
  public void update() {
    if (round != null) {
      //Sort players according to rating
      players = round.getTournament().getPlayers();
      Collections.sort(players);
    }
    else {
      players = null;
    }
    initColumnNames();
    fireTableDataChanged();
  }

  void initColumnNames() {
    columnNames.clear();
    columnNames.add("Bord");
    columnNames.add("Vit spelare");
    if (round != null && round.getTournament().getShowELO()) {
      columnNames.add("Rating");
    }
    columnNames.add("Resultat");
    resultColumn = columnNames.size()-1;
    columnNames.add("Svart spelare");
    if (round != null && round.getTournament().getShowELO()) {
      columnNames.add("Rating");
    }
    this.fireTableStructureChanged();
  }

  int getResultColumn() {
    return resultColumn;
  }
}
