package cp.gui;

import cp.Tournament;
import java.awt.event.*;
import java.awt.print.PrinterException;
import java.text.MessageFormat;
import java.util.*;
import javax.swing.*;
import javax.swing.table.*;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Text;

public class PlayerListPanel extends JScrollPane {
  private TournamentPlayerListTableModel tableModel;
  private JTable jTable, jPrintTable;

  public PlayerListPanel() {
    try {
      tableModel = new TournamentPlayerListTableModel();
      jTable = new JTable(tableModel);
      jPrintTable = new JTable(tableModel);
      jbInit();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  private void jbInit() throws Exception {
    this.setViewportView(jTable);
    jTable.setFillsViewportHeight(true);
    setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
    setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
    this.addComponentListener(new ComponentAdapter() {
      public void componentShown(ComponentEvent e) {
        this_componentShown();
      }
    });
    jTable.setShowGrid(false);
    jTable.setShowHorizontalLines(true);
    jPrintTable.setShowGrid(false);
    jPrintTable.setShowHorizontalLines(true);
  }

  void setTournament(Tournament tournament) {
    tableModel.setTournament(tournament);
    // Setup column sorting
    TableRowSorter<TournamentPlayerListTableModel> sorter = new TableRowSorter<TournamentPlayerListTableModel>(tableModel);
    jTable.setRowSorter(sorter);
    jPrintTable.setRowSorter(sorter);
    List<RowSorter.SortKey> sortKeys = new ArrayList<RowSorter.SortKey>();
    sortKeys.add(new RowSorter.SortKey(1, SortOrder.ASCENDING)); // Namn
    if (tableModel.getTournament() != null && tableModel.getTournament().getShowGroup()) {
      sortKeys.add(new RowSorter.SortKey(3, SortOrder.ASCENDING)); // Klass
    }
    else {
      sortKeys.add(new RowSorter.SortKey(2, SortOrder.ASCENDING)); // Klass
    }   
    sorter.setSortKeys(sortKeys);
  }

  void print() throws PrinterException {
    Tournament tournament = tableModel.getTournament();
    if (tournament != null) {
      MessageFormat headerFormat = new MessageFormat(String.format("Spelare i turnering %s, grupp %s", tournament, tournament.getGroup()));
      EditResultsPanel.optimizeTableWidth(jPrintTable);
      jPrintTable.print(JTable.PrintMode.FIT_WIDTH, headerFormat, null);
    }
  }

  public void save(Document document, Element body) {
//    JOptionPane.showMessageDialog(this, "Lottningen publiceras sorterad p " + jTable.getColumnName(sorter.getSortKeys().get(0).getColumn()), "Information", JOptionPane.INFORMATION_MESSAGE);
    // Insert table
    Element table = document.createElement("table");
    body.appendChild(table);
    table.setAttribute("class", "CP_TablePlayers");
    table.setAttribute("cellpadding", "0");
    table.setAttribute("cellspacing", "0");
    table.setAttribute("border", "0");
    // Create headers
    Element tr = document.createElement("tr");
    tr.setAttribute("class", "CP_TablePlayersHeader");
    table.appendChild(tr);
    for (int i = 0; i < jTable.getColumnCount(); i++) {
      String header = jTable.getColumnName(i);
      Element th = document.createElement("th");
      tr.appendChild(th);
      th.setAttribute("class", String.format("CP_TablePlayersHeader%s", header));
      Text text = document.createTextNode(header);
      th.appendChild(text);
    }
    for (int i = 0; i < jTable.getRowCount(); i++) {
      tr = document.createElement("tr");
      table.appendChild(tr);
      tr.setAttribute("class", "CP_TablePlayersRow");
      for (int j = 0; j < jTable.getColumnCount(); j++) {
        String header = jTable.getColumnName(j);
        Element th = document.createElement("th");
        tr.appendChild(th);
        th.setAttribute("class", String.format("CP_TablePlayers%s", header));
        Object obj = jTable.getValueAt(i, j);
        if (obj != null) {
          Text text = document.createTextNode(obj.toString());
          th.appendChild(text);
        }
      }
    }
  }

  public Tournament getTournament() {
    return tableModel.getTournament();
  }

  public void update() {
    if (tableModel != null) {
      tableModel.update();
      tableModel.setColumnWidths(jTable);
      tableModel.setColumnWidths(jPrintTable);
    }
    jTable.doLayout();
    jTable.repaint();
  }

  private void this_componentShown() {
    update();
  }
}
